<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 2846022b1c50 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-06-27 12:58 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_cache_backendMemcache implements we_cache_backend{
	/** @var Memcached */
	private $cache;

	public function __construct(string $id){
		$this->cache = new Memcached(/* $id */);
		$this->cache->setOptions([
			Memcached::OPT_BINARY_PROTOCOL => true,
			Memcached::OPT_PREFIX_KEY => $id . '_',
			Memcached::OPT_COMPRESSION => true,
			Memcached::OPT_CONNECT_TIMEOUT => 30,
			Memcached::OPT_DISTRIBUTION => Memcached::DISTRIBUTION_CONSISTENT,
			Memcached::OPT_SERVER_FAILURE_LIMIT => 2,
			Memcached::OPT_REMOVE_FAILED_SERVERS => true,
			Memcached::OPT_RETRY_TIMEOUT => 60,
		]);
		if($this->cache->isPristine() || count($this->cache->getServerList()) != count(MEMCACHE_SERVERS)){
			$this->setServers();
		}
	}

	/**
	 *
	 * @param string[] $servers
	 * @return void
	 */
	public function setServers(array $servers = []): void{
		$this->cache->resetServerList();
		$arr = [];
		foreach(($servers ?: MEMCACHE_SERVERS) as $serv){
			$server = explode(':', $serv);
			$arr[] = [$server[0], (int) ($server[1] ?? 0), (int) ($server[2] ?? 0)];
		}
		$this->cache->addServers($arr);
	}

	public function clean(string $hash = ''): bool{
		return $this->cache->delete($hash);
	}

	/**
	 *
	 * @param string $entry
	 * @return mixed
	 * @throws we_exception_cache
	 */
	public function get(string $entry){
		if(!$this->cache){
			throw new we_exception_cache();
		}
		$data = $this->cache->get($entry);
		if($data === false){
			throw new we_exception_cache();
		}
		return $data;
	}

	public function set(string $entry, $data, int $expiry): bool{
		if(is_string($data) && strlen($data) > 512 * 1024){
			$this->cache->delete($entry);
			return false;
		}
		$ret = $this->cache->set($entry, $data, $expiry);
		if(!$ret){
			t_e('warning', $entry, strlen(is_string($data) ? $data : serialize($data)), $this->cache->getResultMessage(), $this->cache->getServerList());
		}
		return $ret;
	}

	public function touch(string $entry, int $expiry = 3600): bool{
		return $this->cache->touch($entry, $expiry);
	}

	public function increment(string $entry, int $default = 1, int $expiry = 3600): int{
		return (int) $this->cache->increment($entry, 1, $default, $expiry);
	}

	public function quit(): void{
		$this->cache->quit();
	}

	public function getStatus(): string{
		$fields = ['Server' => '', 'Current Items' => 'curr_items', 'Total Items' => 'total_items', 'Connections' => 'curr_connections', 'Version' => 'version'];
		$status = $this->cache->getStats();
		'@phan-var array<string,mixed>|false $status';
		if($status === false){
			return '-';
		}
		$ret = new we_html_table(['class' => 'default statusTable'], count($status), count($fields));
		$ret->addHeaderRow(array_keys($fields));
		$rowid = 1;
		array_shift($fields);
		foreach($status as $name => $stat){
			$colid = 0;
			$ret->setCol($rowid, $colid++, $name);
			foreach($fields as $col){
				$ret->setCol($rowid, $colid++, (string) $stat[$col]);
			}
			$rowid++;
		}
		return $ret->getHtml();
	}

}
