<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 67a3e1c07927 $
 * $Author: Lukas Imhof $
 * $Date: 2021-09-02 17:27 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_glossary_replace{
	const NO_GLOSSAR_TAG = 'no-glossar';

	public static function useAutomatic(): bool{
		return (bool) (fS('SELECT pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="glossary" AND pref_name="GlossaryAutomaticReplacement"', '1'));
	}

	/**
	 * replace the content
	 *
	 * @param string $content
	 * @param string $language
	 */
	public static function replace(string $content, string $language): string{
		return (self::useAutomatic() ?
			self::doReplace($content, $language) :
			$content);
	}

	/**
	 * replace all glossary items for the requested language in the
	 * given source code
	 *
	 * @param string $src
	 * @param string $language
	 * @return string
	 */
	public static function doReplace(string $src, string $language): string{
		if(!$language){
			$language = we_base_WE::$Langs['DefaultFrontendLanguage'];
		}
		$matches = [];
		// get the words to replace
		$cache = new we_glossary_cache($language);
		$replace = [
			'' => $cache->get(we_glossary_glossary::TYPE_TEXTREPLACE), //text replacement must come first, since other might generate irritation annotations
			'span' => $cache->get(we_glossary_glossary::TYPE_FOREIGNWORD),
			'abbr' => (REPLACEACRONYM ? array_merge($cache->get(we_glossary_glossary::TYPE_ABBREVATION), $cache->get(we_glossary_glossary::TYPE_ACRONYM)) : $cache->get(we_glossary_glossary::TYPE_ABBREVATION)),
			'acronym' => (REPLACEACRONYM ? [] : $cache->get(we_glossary_glossary::TYPE_ACRONYM)),
			'a' => $cache->get(we_glossary_glossary::TYPE_LINK),
		];

		unset($cache);
		$lowDocPath = strtolower(we_base_WE::getWeDoc()->Path);
		//forbid self-reference links
		foreach($replace['a'] as $k => $rep){
			if(strpos(strtolower((string) $rep), 'href="' . $lowDocPath) !== false){
				unset($replace['a'][$k]);
			}
		}

		//remove empty elements
		foreach($replace as $tag => $words){
			if(!$words){
				unset($replace[$tag]);
			}
		}

		// first check if there is a body tag inside the sourcecode
		preg_match('|<body[^>]*>(.*)</body>|si', $src, $matches);

		$srcBody = $replBody = ($matches[1] ?? $src);
		/*
		  This is the fastest variant
		 */
		// split the source into tag and non-tag pieces
		$pieces = preg_split('|(<[^>]*>)|', $replBody, -1, PREG_SPLIT_DELIM_CAPTURE | PREG_SPLIT_NO_EMPTY);
		// replace words in non-tag pieces
		$lastHtmlTag = '';
		$tagMatch = [];
		$ignoreTags = ['script' => 0, 'style' => 0, 'textarea' => 0, 'select' => 0, 'abbr' => 0, 'acronym' => 0, self::NO_GLOSSAR_TAG => 0];
		foreach($pieces as &$piece){
			if(preg_match('|^<(/)?([[:alnum:]-]+)|', $piece, $tagMatch)){//is a tag
				[, $not, $lastHtmlTag] = $tagMatch;
				$lastHtmlTag = strtolower($lastHtmlTag);
				if(isset($ignoreTags[$lastHtmlTag])){
					$ignoreTags[$lastHtmlTag] += ($not ? -1 : 1);
					if($lastHtmlTag === self::NO_GLOSSAR_TAG){
						$piece = '';
					}
				}
			} elseif(!array_sum($ignoreTags)){//only if no ignored tag is open!
				//this will generate invalid code: $piece = str_replace('&quot;', '"', $piece);
				foreach($replace as $tag => $words){
					if(!$tag || $lastHtmlTag !== $tag){
						$piece = self::doReplaceWords($piece, $words);
					}
				}
			}
		}

		$replBody = strtr(implode('', $pieces), ['@@@we@@@' => '\'']);
		return (isset($matches[1]) ?
			str_replace($srcBody, $replBody, $src) :
			$replBody);
	}

	/**
	 * replace just the given replacements in the given source
	 *
	 * @param string $src
	 * @param array<string,string> $replacements
	 * @return string
	 */
	private static function doReplaceWords(string $src, array $replacements): string{
		if($src === '' || !$replacements){
			return $src;
		}
		return preg_replace(array_keys($replacements), $replacements, $src);
	}

}
