<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 785c2a488c30 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-02-25 17:57 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * Implements JWT encoding and decoding as per http://tools.ietf.org/html/draft-ietf-oauth-json-web-token-06
 * Encoding algorithm based on http://code.google.com/p/google-api-php-client
 * Decoding algorithm based on https://github.com/luciferous/jwt
 * Src: https://github.com/F21/jwt/blob/master/JWT/JWT.php
 * @author Francis Chuang <francis.chuang@gmail.com>
 */
abstract class we_helpers_jwt{
	const ALG_HS256 = 'HS256';
	const ALG_HS384 = 'HS384';
	const ALG_HS512 = 'HS512';
	const ALG_RS256 = 'RS256';
	const ALG_RS384 = 'RS384';
	const ALG_RS512 = 'RS512';

	/**
	 *
	 * @param array $payload
	 * @param string $key
	 * @param string $algo
	 * @return string
	 * @throws we_exception_base
	 */
	public static function encode(array $payload, string $key, string $algo = self::ALG_HS256): string{
		$header = ['typ' => 'JWT', 'alg' => $algo];
		try{
			$segments = [
				self::urlsafeB64Encode((json_encode($header, defined('JSON_THROW_ON_ERROR') ? JSON_THROW_ON_ERROR : 0) ?: '')), //FIX PHP 7.3
				self::urlsafeB64Encode((json_encode($payload, defined('JSON_THROW_ON_ERROR') ? JSON_THROW_ON_ERROR : 0) ?: ''))
			];
		} catch (Exception $e){
			throw new we_exception_base($e->getMessage());
		}
		$signing_input = implode('.', $segments);
		$signature = self::sign($signing_input, $key, $algo);
		$segments[] = self::urlsafeB64Encode($signature);
		return implode('.', $segments);
	}

	/**
	 *
	 * @param string $jwt
	 * @param string $key
	 * @param string $algo
	 * @return string
	 * @throws we_exception_base
	 * @throws UnexpectedValueException
	 */
	public static function decode(string $jwt, string $key = '', string $algo = self::ALG_HS256): array{
		$tks = explode('.', $jwt);

		if(count($tks) != 3){
			throw new we_exception_base('Wrong number of segments');
		}

		[$headb64, $payloadb64, $cryptob64] = $tks;
		$header = json_decode(self::urlsafeB64Decode($headb64));
		if($header === null){
			throw new we_exception_base('Invalid segment encoding');
		}
		$payload = json_decode(self::urlsafeB64Decode($payloadb64), true);
		if($payload === null){
			throw new we_exception_base('Invalid segment encoding');
		}

		$sig = self::urlsafeB64Decode($cryptob64);

		if($key){
			if(empty($header->alg)){
				throw new we_exception_base('Empty algorithm');
			}

			if(!self::verifySignature($sig, "$headb64.$payloadb64", $key, $algo)){
				throw new UnexpectedValueException('Signature verification failed');
			}
		}
		assert(is_array($payload));
		return $payload;
	}

	/**
	 *
	 * @param string $signature
	 * @param string $input
	 * @param string $key
	 * @param string $algo
	 * @return bool
	 * @throws Exception
	 * @throws we_exception_base
	 */
	private static function verifySignature(string $signature, string $input, string $key, string $algo): bool{
		switch($algo){
			case self::ALG_HS256:
			case self::ALG_HS384:
			case self::ALG_HS512:
				return self::sign($input, $key, $algo) === $signature;
			case self::ALG_RS256:
				return (boolean) openssl_verify($input, $signature, $key, OPENSSL_ALGO_SHA256);
			case self::ALG_RS384:
				return (boolean) openssl_verify($input, $signature, $key, OPENSSL_ALGO_SHA384);
			case self::ALG_RS512:
				return (boolean) openssl_verify($input, $signature, $key, OPENSSL_ALGO_SHA512);
			default:
				throw new we_exception_base("Unsupported or invalid signing algorithm.");
		}
	}

	/**
	 *
	 * @param string $input
	 * @param string $key
	 * @param string $algo
	 * @return string
	 * @throws we_exception_base
	 */
	private static function sign(string $input, string $key, string $algo): string{
		switch($algo){
			case self::ALG_HS256:
				return hash_hmac('sha256', $input, $key, true);
			case self::ALG_HS384:
				return hash_hmac('sha384', $input, $key, true);
			case self::ALG_HS512:
				return hash_hmac('sha512', $input, $key, true);
			case self::ALG_RS256:
				return self::generateRSASignature($input, $key, OPENSSL_ALGO_SHA256);
			case self::ALG_RS384:
				return self::generateRSASignature($input, $key, OPENSSL_ALGO_SHA384);
			case self::ALG_RS512:
				return self::generateRSASignature($input, $key, OPENSSL_ALGO_SHA512);
			default:
				throw new we_exception_base("Unsupported or invalid signing algorithm.");
		}
	}

	/**
	 *
	 * @param string $input
	 * @param string $key
	 * @param int $algo
	 * @return string
	 * @throws we_exception_base
	 */
	private static function generateRSASignature(string $input, string $key, int $algo): string{
		$signature = '';
		if(!openssl_sign($input, $signature, $key, $algo)){
			throw new we_exception_base("Unable to sign data.");
		}

		return $signature;
	}

	public static function urlSafeB64Encode(string $data): string{
		return str_replace(['+', '/', '\r', '\n', '='], ['-', '_'], base64_encode($data));
	}

	public static function urlSafeB64Decode(string $b64): string{
		return base64_decode(strtr($b64, ['-' => '+', '_' => '/'])) ?: '';
	}

}
