<?php

declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 03dc86d0ae17 $
 * $Author: Lukas Imhof $
 * $Date: 2021-08-05 18:17 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_userInput extends we_weTag_tag{

	/** @var bool */
	private static $property = false;

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag
	 * @param bool $internal if used by we internally @unused-param
	 * @return string output to print
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		self::attributeMissing($attribs, ['name' => false], __CLASS__);
		self::$doc = we_base_WE::getWeDoc();

		$name = self::getAttribute('name', $attribs, '', we_base_request::STRING);
		$type = self::getAttribute('type', $attribs, '', we_base_request::STRING);
		self::$property = (bool) self::getAttribute('property', $attribs, false, we_base_request::BOOL);
		$checked = (bool) self::getAttribute('checked', $attribs, false, we_base_request::BOOL);
		$value = self::getAttribute('value', $attribs, '', we_base_request::STRING);
		$editable = (bool) self::getAttribute('editable', $attribs, true, we_base_request::BOOL);
		$hidden = (bool) self::getAttribute('hidden', $attribs, false, we_base_request::BOOL);
		$size = self::getAttribute('size', $attribs, '', we_base_request::UNIT);
		$values = self::getAttribute('values', $attribs, [], we_base_request::STRING_LIST);
		$removeFirstParagraph = (bool) self::getAttribute('removefirstparagraph', $attribs, defined('REMOVEFIRSTPARAGRAPH_DEFAULT') ? REMOVEFIRSTPARAGRAPH_DEFAULT : true, we_base_request::BOOL);

		if($hidden && ($type != 'date')){
			$type = 'hidden';
		}

		$object = (we_tag_form::$name ?
				(we_tag_write::$objElements[we_tag_form::$name] ??
				we_tag_write::$docElements[we_tag_form::$name] ??
				self::$doc) :
				null);

		if($object){
			if($object instanceof we_contents_objectFile){
				$name = self::getAttribute('_name_orig', $attribs, '', we_base_request::STRING);
				$tableID = $object->TableID;
			} else {
				$tableID = 0;
			}
			if(self::$property){
				$isset = isset($object->{$name});
				$orgVal = $isset ? $object->{$name} : $value;
			} else {
				$orgVal = 0;
				$isempty = true;
				if($object->issetElement($name)){
					switch($object->getElementAccess($name)){
						case we_contents_base::ACCESS_B:
							$isempty = !($orgVal = $object->getElementB($name));
							break;
						case we_contents_base::ACCESS_I:
							$isempty = ($orgVal = $object->getElementI($name)) === 0;
							break;
						case we_contents_base::ACCESS_F:
							$isempty = $object instanceof we_contents_objectFile ? ($orgVal = $object->getElementF($name)) === 0.0 : true;
							break;
						case we_contents_base::ACCESS_S:
							$isempty = ($orgVal = $object->getElementS($name)) === '';
							break;
						// IMPORTANT: no ACCESS_A elements possible here!
					}
				}
				$isset = !$object->ID && $isempty ? false : $object->issetElement($name);
				$orgVal = $isset ? (string) $orgVal : $value;
			}
//FIXME e.g. data passed to img tag contains size, quality
			$content = $object->getFieldByVal($orgVal, $type, $attribs, true, $object->ParentID, $object->Path, $tableID);
		} else {
			$orgVal = $value;
			$isset = false;
		}

		$fieldname = 'we_ui_' . (we_tag_form::$name ?: '') .
				(self::$property ? ( '_' . $name) : ('[' . $name . ']') );

		if(!$editable && !$hidden){
			switch($type){
				case 'img':
				case 'binary':
				case 'video':
				case 'audio':
					break;
				default:
					$hidden = self::htmlTag('input', [
								'type' => 'hidden',
								'name' => $fieldname,
								'value' => htmlspecialchars($orgVal),
					]);
					return (($type != 'hidden') ? $content : '') . $hidden;
			}
		}
		switch($type){
			case 'img' :
				$imgDataId = we_base_request::string(we_fileupload::USERINPUT_TYPE_IMG . $name, md5(uniqid(__CLASS__, true)));
				return $editable ?
						self::getImgEdit($attribs, $imgDataId, $name, $orgVal, $fieldname, $size) :
						self::getImg($attribs, $imgDataId, $name, $orgVal);
			case 'audio':
				return self::getAudio($attribs, $editable, $orgVal, $size, $fieldname, $name);
			case 'video':
				return self::getVideo($attribs, $editable, $orgVal, $size, $fieldname, $name);
			case 'binary' :
				return self::getBinary($attribs, $editable, $orgVal, $size, $fieldname, $name);
			case 'textarea' :
				$data = (string) ($content ?: $value);
				if(self::getAttribute('wysiwyg', $attribs, false, we_base_request::BOOL)){
					return we_html_forms::weTextarea($fieldname, $data, $attribs, false, false, $removeFirstParagraph, true, $name, self::$doc->getDocumentCss());
				}
				$atts = self::removeAttribs($attribs, [
							'_name_orig',
							'autobr',
							'bgcolor',
							'checked',
							'commands',
							'editable',
							'editorcss',
							'fontnames',
							'format',
							'hidden',
							'hideautobr' .
							'ignoredocumentcss',
							'maxlength',
							'property',
							'pure',
							'size',
							'type',
							'value',
							'values',
							'wysiwyg',
				]);
				$atts['name'] = $fieldname;
				$atts['class'] = (isset($atts['class']) ? $atts['class'] . ' ' : '') . 'we we_userinput';

				return self::htmlTag('textarea', $atts, htmlspecialchars($data), true);
			case 'checkbox' :
				$atts = self::removeAttribs($attribs, ['wysiwyg', 'commands', 'pure', 'type', 'value', 'checked', 'autobr', 'name', 'values',
							'hidden', 'editable', 'format', 'property', 'cols', 'rows', 'width', 'height', 'bgcolor', 'fontnames']);

				$atts['class'] = (isset($atts['class']) ? $atts['class'] . ' ' : '') . 'we we_userinput';
				return self::inputCheckboxField($fieldname, ((!$isset) && $checked ? '1' : (string) $content), $atts);
			case 'date' :
				return self::getDate($attribs, $hidden, $fieldname, $orgVal, $name);
			case 'country':
				return self::getCountry($attribs, $orgVal, $fieldname);
			case 'language':
				return self::getLanguage($attribs, $orgVal, $fieldname);
			case 'select' :
				$options = '';
				$atts = self::removeAttribs($attribs, ['wysiwyg', 'commands', 'pure', 'type', 'value', 'checked', 'autobr', 'name', 'values',
							'hidden', 'editable', 'format', 'property', 'rows', 'cols', 'fontnames', 'bgcolor', 'width', 'height', 'maxlength'
				]);
				if($values){
					foreach($values as $txt){
						$options .= self::htmlTag('option', ($txt == $orgVal ? ['selected' => 'selected'] : []), trim($txt), true);
					}
				} elseif($object && !empty($object->DefArray['select_' . $name])){
					foreach(array_combine($object->DefArray['select_' . $name]['selectKeys'], $object->DefArray['select_' . $name]['selectValues']) as $key => $val){
						$atts2 = ($orgVal == $key ? ['value' => $key, 'selected' => 'selected'] : ['value' => $key]);
						$attsOption = self::removeAttribs(array_merge($atts, $atts2), ['class']);
						$options .= self::htmlTag('option', $attsOption, $val, true);
					}
				}
				$atts['size'] = (empty($atts['size']) ? 1 : $atts['size']);
				$atts['name'] = $fieldname;
				return we_html_element::span(['class' => 'we we_userinput'], self::htmlTag('select', $atts, $options, true));
			case 'radio' :
				$atts = self::removeAttribs($attribs, ['wysiwyg', 'commands', 'pure', 'type', 'value', 'checked', 'autobr', 'name', 'values', 'hidden',
							'editable', 'format', 'property', 'rows', 'cols', 'width', 'height', 'bgcolor', 'fontnames'
				]);
				$atts['class'] = (isset($atts['class']) ? $atts['class'] . ' ' : '') . 'we we_userinput';
				return (!$isset ?
						self::inputRadioField($fieldname, ($checked ? $value : $value . 'dummy'), $value, $atts) :
						self::inputRadioField($fieldname, $orgVal, $value, $atts));

			case 'hidden':
				return self::htmlTag('input', [
							'type' => 'hidden',
							'name' => $fieldname,
							'value' => htmlspecialchars(self::getAttribute('value', $attribs, $content, we_base_request::STRING)),
				]);
			case 'print' :
				return $orgVal;
			case 'choice' :
				$atts = self::removeAttribs($attribs, ['wysiwyg', 'commands', 'pure', 'type', 'value', 'checked', 'autobr', 'name', 'values',
							'hidden', 'editable', 'format', 'property', 'cols', 'rows', 'width', 'height', 'bgcolor', 'fontnames', 'maxlength'
				]);
				$mode = self::getAttribute('mode', $attribs, '', we_base_request::STRING);
				return we_html_element::span(['class' => 'we we_userinput'], we_html_tools::inputChoiceField($fieldname, $orgVal, $values, $atts, $mode));
			case 'password':
				$atts = array_merge(self::removeAttribs($attribs, ['wysiwyg', 'commands', 'pure', 'type', 'value', 'checked', 'autobr', 'name', 'values', 'hidden', 'editable',
							'format',
							'property', 'cols', 'rows', 'width', 'height', 'bgcolor', 'fontnames']), ['type' => 'password', 'name' => $fieldname, 'value' => htmlspecialchars($orgVal)]
				);
				$atts['class'] = (isset($atts['class']) ? $atts['class'] . ' ' : '') . 'we we_userinput';

				return self::htmlTag('input', $atts);
			case 'textinput':
			default:
				$atts = self::removeAttribs($attribs, ['wysiwyg', 'commands', 'pure', 'type', 'value', 'checked', 'autobr', 'name', 'values', 'hidden', 'editable', 'format',
							'property',
							'cols', 'rows', 'width', 'height', 'bgcolor', 'fontnames']);
				$atts['class'] = (isset($atts['class']) ? $atts['class'] . ' ' : '') . 'we we_userinput';
				return self::inputTextInputField($fieldname, $orgVal, $atts);
		}
	}

	protected static function getImg(array $attribs, string $imgDataId, string $name, string $orgVal): string{
		$hidden = '<input type="hidden" name="' . we_fileupload::USERINPUT_TYPE_IMG . $name . '" value="' . $imgDataId . '" />';
		if(($serverPath = $_SESSION[$imgDataId]['serverPath'] ?? '')){
			return self::htmlTag('img', [
						'src' => we_base_file::addPreSlash(substr($serverPath, strlen(WE_DOCUMENT_ROOT)) ?: ''),
						'alt' => '',
						'style' => 'width:' . $_SESSION[$imgDataId]['imgwidth'] . 'px;height:' . $_SESSION[$imgDataId]["imgheight"] . 'px',
					]) . $hidden;
		}

		if(!empty($_SESSION[$imgDataId]['doDelete'])){
			return $hidden;
		}

		if((!empty($_SESSION[$imgDataId]['id'])) || $orgVal){//Fix #9835
			unset($attribs['width'], $attribs['height']);
			$attribs['id'] = $_SESSION[$imgDataId]['id'] ?? $orgVal;
			return self::$doc->getField(array_intersect_key($attribs, ['showcontrol' => false, 'id' => 0, 'name' => '', 'alt' => '', 'title' => '', 'thumbnail' => '', 'thumbnailid' => 0] + we_document_image::ELEMENT_KEY), 'img') . $hidden;
		}

		return '';
	}

	protected static function getImgEdit(array $attribs, string $imgDataId, string $name, string $orgVal, string $fieldname, string $size): string{
		self::attributeMissing($attribs, ['parentid' => false], __CLASS__);

		if(!isset($_SESSION[$imgDataId])){
			$_SESSION[$imgDataId] = [];
		}
		$_SESSION[$imgDataId]['parentid'] = self::getAttribute('parentid', $attribs, 0, we_base_request::INT);
		//$_SESSION[$imgDataId]['maxfilesize'] = sefl::getAttribute('maxfilesize',$attribs);
		$_SESSION[$imgDataId]['width'] = self::getAttribute('width', $attribs, 0, we_base_request::INT);
		$_SESSION[$imgDataId]['height'] = self::getAttribute('height', $attribs, 0, we_base_request::INT);
		$_SESSION[$imgDataId]['quality'] = self::getAttribute('quality', $attribs, 8, we_base_request::INT);
		$_SESSION[$imgDataId]['keepratio'] = self::getAttribute('keepratio', $attribs, true, we_base_request::BOOL);
		$_SESSION[$imgDataId]['maximize'] = self::getAttribute('maximize', $attribs, false, we_base_request::BOOL);
		$_SESSION[$imgDataId]['id'] = $orgVal ?: '';

		$bordercolor = self::getAttribute('bordercolor', $attribs, '#006DB8', we_base_request::STRING);
		$checkboxstyle = self::getAttribute('checkboxstyle', $attribs, '', we_base_request::STRING);
		$inputstyles = self::getAttribute('inputstyle', $attribs, '', we_base_request::STRING);
		$checkboxclass = self::getAttribute('checkboxclass', $attribs, '', we_base_request::STRING);
		$inputclass = self::getAttribute('inputclass', $attribs, '', we_base_request::STRING);
		$checkboxtext = self::getAttribute('checkboxtext', $attribs, g_l('parser', '[delete]'), we_base_request::STRING);

		if($_SESSION[$imgDataId]['id']){
			$attribs['id'] = $_SESSION[$imgDataId]['id'];
		}

		if(isset($_SESSION[$imgDataId]['serverPath']) && !str_contains($_SESSION[$imgDataId]['serverPath'], TEMP_PATH)){
			$imgTag = self::htmlTag('img', [
						'src' => we_base_file::addPreSlash(substr($_SESSION[$imgDataId]['serverPath'], strlen(WE_DOCUMENT_ROOT)) ?: ''),
						'alt' => ' ',
						'width' => $_SESSION[$imgDataId]['imgwidth'],
						'height' >= $_SESSION[$imgDataId]['imgheight'],
			]);
		} else {
			unset($attribs['width'], $attribs['height']);
			$imgTag = self::$doc->getField(array_intersect_key($attribs, ['showcontrol' => false, 'id' => 0, 'name' => '', 'alt' => '', 'title' => '', 'thumbnail' => '', 'thumbnailid' => 0] + we_document_image::ELEMENT_KEY), 'img');
		}

		$checked = (!empty($_SESSION[$imgDataId]['doDelete'])) ? ' checked' : '';
		$inputstyle = ($size ? 'width:' . $size . 'em;' . $inputstyles : $inputstyles);
		return '<table class="we we_userinput weEditTable" style="border-color:' . $bordercolor . ';">
	<tr>
		<td class="weEditmodeStyle" colspan="2" style="text-align:center">' . $imgTag . '
			<input type="hidden" name="' . we_fileupload::USERINPUT_TYPE_IMG . $name . '" value="' . $imgDataId . '" /></td>
	</tr>
	<tr>
		<td class="weEditmodeStyle" colspan="2" style="text-align:left">
			<input name="' . $fieldname . '" type="file" accept="' . implode(',', we_base_ContentTypes::getRealContentTypes(we_base_ContentTypes::IMAGE)) . '"' . ($inputstyle ? (' style="' . $inputstyle . '"') : '') . ($inputclass ? (' class="' . $inputclass . '"') : '') . '/>
		</td>
	</tr>
	<tr>
		<td class="weEditmodeStyle" colspan="2" style="text-align:left">
			<table class="weEditTable">
				<tr>
					<td style="padding-right: 5px;"><input style="border:0 solid black;" type="checkbox" id="' . we_fileupload::USERINPUT_TYPE_DEL . $name . '" name="' . we_fileupload::USERINPUT_TYPE_DEL . $name . '" value="1" ' . $checked . '/></td>
					<td><label for="' . we_fileupload::USERINPUT_TYPE_DEL . $name . '"' . ($checkboxstyle ? (' style="' . $checkboxstyle . '"') : '') . ($checkboxclass ? (' class="' . $checkboxclass . '"') : '') . '>' . $checkboxtext . '</label></td>
				</tr>
			</table>
		</td>
	</tr>
</table>';
	}

	protected static function getDate(array $attribs, bool $hidden, string $fieldname, string $orgVal, string $name): string{
		if($hidden){
			$attsHidden = ['type' => 'hidden',
				'name' => $fieldname,
				'value' => $orgVal ?: time(),
			];
			return self::htmlTag('input', $attsHidden);
		}
		$currentdate = self::getAttribute('currentdate', $attribs, false, we_base_request::BOOL);
		$minyear = self::getAttribute('minyear', $attribs, 0, we_base_request::INT);
//		$format = self::getAttribute('format', $attribs, '', we_base_request::STRING);

		$orgVal = $orgVal ?: ($currentdate ? time() : 0);
		$year = (int) date('Y');
		switch($minyear ? $minyear[0] : ''){
			case '+':
				$minyear = $year + intval(substr($minyear, 1));
				break;
			case '-':
				$minyear = $year - intval(substr($minyear, 1));
				break;
			default:
				$minyear = intval($minyear);
				break;
		}
		$maxyear = self::getAttribute('maxyear', $attribs, 0, we_base_request::INT);
		switch($maxyear ? $maxyear[0] : ''){
			case '+':
				$maxyear = $year + intval(substr($maxyear, 1));
				break;
			case '-':
				$maxyear = $year - intval(substr($maxyear, 1));
				break;
			default:
				$maxyear = intval($maxyear);
				break;
		}
		return we_html_element::span(['class' => 'we we_userinput'],
						we_html_tools::getNewDateInput('we_ui_' . (we_tag_form::$name ?: '') . '[we_date_' . $name . '][%s]', new DateTime((date('Y-m-d H:i:s', (int) $orgVal))), we_html_tools::FORMAT_DATE_TIME, false, false, $minyear ? new DateTime($minyear . '-01-01') : null, $maxyear ? new DateTime($maxyear . '-12-31') : null));
	}

	protected static function getLanguage(array $attribs, string $orgVal, string $fieldname): string{
		$docAttr = self::getAttribute('doc', $attribs, 'self', we_base_request::STRING);

		$newAtts = self::removeAttribs($attribs, ['wysiwyg', 'commands', 'pure', 'type', 'value', 'checked', 'autobr', 'name', 'values', 'hidden', 'editable', 'format',
					'property',
					'rows', 'cols', 'fontnames', 'bgcolor', 'width', 'height', 'maxlength']);

		$doc = self::getDocForTag($docAttr);
		$lang = $doc->Language;
		$langcode = we_tag_input::getLangCode($attribs);

		$frontendL = we_base_WE::$Langs['FrontendLanguages'];
		foreach($frontendL as &$lcvalue){
			$lccode = explode('_', $lcvalue);
			$lcvalue = $lccode[0];
		}
		$frontendLL = [];
		foreach($frontendL as &$lcvalue){
			$frontendLL[$lcvalue] = we_base_country::getTranslation($lcvalue, we_base_country::LANGUAGE, $langcode);
		}

		$oldLocale = setlocale(LC_ALL, NULL);
		setlocale(LC_ALL, $lang . '.UTF-8');
		asort($frontendLL, SORT_LOCALE_STRING);
		if($oldLocale){
			setlocale(LC_ALL, $oldLocale);
		}
		$options = '';
		foreach($frontendLL as $langkey => &$langvalue){
			$options .= '<option value="' . $langkey . '" ' . ($orgVal == $langkey ? ' selected="selected">' : '>') . $langvalue . '</option>';
		}
		unset($langvalue);
		$newAtts['size'] = ($attribs['size'] ?? 1);
		$newAtts['name'] = $fieldname;
		$newAtts['class'] = (isset($newAtts['class']) ? $newAtts['class'] . ' ' : '') . 'we we_userinput';
		return self::htmlTag('select', $newAtts, $options, true);
	}

	protected static function getCountry(array $attribs, string $orgVal, string $fieldname): string{
		$newAtts = self::removeAttribs($attribs, ['wysiwyg', 'commands', 'pure', 'type', 'value', 'checked', 'autobr', 'name', 'values', 'hidden', 'editable', 'format',
					'property',
					'rows', 'cols', 'fontnames', 'bgcolor', 'width', 'height', 'maxlength']);
		$docAttr = self::getAttribute('doc', $attribs, 'self', we_base_request::STRING);

		$doc = self::getDocForTag($docAttr);
		$lang = $doc->Language;
		$langcode = we_tag_input::getLangCode($attribs);


		if(WE_COUNTRIES_TOP){
			$topCountries = array_flip(explode(',', WE_COUNTRIES_TOP));
			foreach($topCountries as $countrykey => &$countryvalue){
				$countryvalue = we_base_country::getTranslation((string) $countrykey, we_base_country::TERRITORY, $langcode);
			}
			unset($countryvalue);
		} else {
			$topCountries = [];
		}
		if(WE_COUNTRIES_SHOWN){
			$shownCountries = array_flip(explode(',', WE_COUNTRIES_SHOWN));
			foreach($shownCountries as $countrykey => &$countryvalue){
				$countryvalue = we_base_country::getTranslation((string) $countrykey, we_base_country::TERRITORY, $langcode);
			}
			unset($countryvalue);
		} else {
			$shownCountries = [];
		}
		$oldLocale = setlocale(LC_ALL, NULL);
		setlocale(LC_ALL, $lang . '.UTF-8');
		asort($topCountries, SORT_LOCALE_STRING);
		asort($shownCountries, SORT_LOCALE_STRING);
		if($oldLocale){
			setlocale(LC_ALL, $oldLocale);
		}

		$options = '';
		if(WE_COUNTRIES_DEFAULT != ''){
			$options .= '<option value="--" ' . ($orgVal === '--' ? ' selected="selected">' : '>') . WE_COUNTRIES_DEFAULT . '</option>';
		}
		foreach($topCountries as $countrykey => &$countryvalue){
			$options .= '<option value="' . $countrykey . '" ' . ($orgVal == $countrykey ? ' selected="selected">' : '>') . $countryvalue . '</option>';
		}
		if(!empty($topCountries) && !empty($shownCountries)){
			$options .= '<option value="-" disabled="disabled">----</option>';
		}
		foreach($shownCountries as $countrykey2 => &$countryvalue2){
			$options .= '<option value="' . $countrykey2 . '" ' . ($orgVal == $countrykey2 ? ' selected="selected">' : '>') . $countryvalue2 . '</option>';
		}
		unset($countryvalue, $countryvalue2);
		$newAtts['size'] = ($attribs['size'] ?? 1);
		$newAtts['name'] = $fieldname;
		$newAtts['class'] = (isset($newAtts['class']) ? $newAtts['class'] . ' ' : '') . 'we we_userinput';
		return self::htmlTag('select', $newAtts, $options, true);
	}

	// FIXME: getAudio(), getVideo() and getBinary() are very similar: make one function
	protected static function getAudio(array $attribs, bool $editable, string $orgVal, string $size, string $fieldname, string $name): string{
		$dataId = we_base_request::string(we_fileupload::USERINPUT_TYPE_AUDIO . $name, md5(uniqid(__CLASS__, true)));
		$bordercolor = self::getAttribute('bordercolor', $attribs, '#006DB8', we_base_request::STRING);
		$checkboxstyle = self::getAttribute('checkboxstyle', $attribs, '', we_base_request::STRING);
		$inputstyle = self::getAttribute('inputstyle', $attribs, '', we_base_request::STRING);
		$checkboxclass = self::getAttribute('checkboxclass', $attribs, '', we_base_request::STRING);
		$inputclass = self::getAttribute('inputclass', $attribs, '', we_base_request::STRING);
		$checkboxtext = self::getAttribute('checkboxtext', $attribs, g_l('parser', '[delete]'), we_base_request::RAW);

		if($editable){
			self::attributeMissing($attribs, ['parentid' => false], __CLASS__);

			if(!isset($_SESSION[$dataId])){
				$_SESSION[$dataId] = [];
			}
			$_SESSION[$dataId]['parentid'] = self::getAttribute('parentid', $attribs, 0, we_base_request::INT);
			$_SESSION[$dataId]['id'] = $orgVal ?: '';

			if($_SESSION[$dataId]['id']){
				$attribs['id'] = $_SESSION[$dataId]['id'];
			}

			$tag = (isset($_SESSION[$dataId]['serverPath']) ?
					'' :
					(!empty($attribs['id']) ? self::$doc->getField($attribs, 'audio') : '')
					);

			$checked = (!empty($_SESSION[$dataId]['doDelete']) ? ' checked' : '');
			$inputstyle = ($size ? 'width:' . $size . 'em;' . $inputstyle : $inputstyle);
			return '<table class="we we_userinput weEditTable" style="border-color:' . $bordercolor . ';">
	<tr>
		<td class="weEditmodeStyle" colspan="2" style="text-align:center">' . $tag . '<input type="hidden" name="' . we_fileupload::USERINPUT_TYPE_AUDIO . $name . '" value="' . $dataId . '" /></td>
	</tr>
	<tr>
		<td class="weEditmodeStyle" colspan="2" style="text-align:left"><input name="' . $fieldname . '" type="file" accept="audio/*"' . ($inputstyle ? (' style="' . $inputstyle . '"') : '') . ($inputclass ? (' class="' . $inputclass . '"') : '') . '/></td>
	</tr>
	<tr>
		<td class="weEditmodeStyle" colspan="2" style="text-align:left">
			<table class="weEditTable">
				<tr>
					<td style="padding-right: 5px;"><input style="border:0 solid black;" type="checkbox" id="' . we_fileupload::USERINPUT_TYPE_DEL . $name . '" name="' . we_fileupload::USERINPUT_TYPE_DEL . $name . '" value="1" ' . $checked . '/></td>
					<td><label for="' . we_fileupload::USERINPUT_TYPE_DEL . $name . '"' . ($checkboxstyle ? (' style="' . $checkboxstyle . '"') : '') . ($checkboxclass ? (' class="' . $checkboxclass . '"') : '') . '>' . $checkboxtext . '</label></td>
				</tr>
			</table>
		</td>
	</tr>
</table>';
		}
		$hidden = '<input type="hidden" name="' . we_fileupload::USERINPUT_TYPE_AUDIO . $name . '" value="' . $dataId . '" />';

		if(isset($_SESSION[$dataId]['serverPath'])){
			return $hidden;
		}
		if(!empty($_SESSION[$dataId]['id'])){

			if(!empty($_SESSION[$dataId]['doDelete'])){
				return $hidden;
			}
			$attribs['id'] = $_SESSION[$dataId]['id'];
			return self::$doc->getField($attribs, 'audio') . $hidden;
		}

		return '';
	}

	protected static function getVideo(array $attribs, bool $editable, string $orgVal, string $size, string $fieldname, string $name): string{
		$dataId = we_base_request::string(we_fileupload::USERINPUT_TYPE_VIDEO . $name, md5(uniqid(__CLASS__, true)));
		$bordercolor = self::getAttribute('bordercolor', $attribs, '#006DB8', we_base_request::STRING);
		$checkboxstyle = self::getAttribute('checkboxstyle', $attribs, '', we_base_request::STRING);
		$inputstyle = self::getAttribute('inputstyle', $attribs, '', we_base_request::STRING);
		$checkboxclass = self::getAttribute('checkboxclass', $attribs, '', we_base_request::STRING);
		$inputclass = self::getAttribute('inputclass', $attribs, '', we_base_request::STRING);
		$checkboxtext = self::getAttribute('checkboxtext', $attribs, g_l('parser', '[delete]'), we_base_request::RAW);

		if($editable){
			self::attributeMissing($attribs, ['parentid' => false], __CLASS__);

			if(!isset($_SESSION[$dataId])){
				$_SESSION[$dataId] = [];
			}
			$_SESSION[$dataId]['parentid'] = self::getAttribute('parentid', $attribs, 0, we_base_request::INT);
			$_SESSION[$dataId]['width'] = self::getAttribute('width', $attribs, 0, we_base_request::INT);
			$_SESSION[$dataId]['height'] = self::getAttribute('height', $attribs, 0, we_base_request::INT);
			$_SESSION[$dataId]['id'] = $orgVal ?: '';

			if($_SESSION[$dataId]['id']){
				$attribs['id'] = $_SESSION[$dataId]['id'];
			}

			$tag = (isset($_SESSION[$dataId]['serverPath']) ?
					'' :
					(!empty($attribs['id']) ?
					self::$doc->getField($attribs, 'video') : '')
					);

			$checked = (!empty($_SESSION[$dataId]['doDelete']) ? ' checked' : '');
			$inputstyle = ($size ? 'width:' . $size . 'em;' . $inputstyle : $inputstyle);
			return '<table class="we we_userinput weEditTable" style="border-color:' . $bordercolor . ';">
		<tr>
	<td class="weEditmodeStyle" colspan="2" style="text-align:center">' . $tag . '<input type="hidden" name="' . we_fileupload::USERINPUT_TYPE_VIDEO . $name . '" value="' . $dataId . '" /></td>
		</tr>
		<tr>
	<td class="weEditmodeStyle" colspan="2" style="text-align:left"><input name="' . $fieldname . '" type="file" accept="video/*"' . ($inputstyle ? (' style="' . $inputstyle . '"') : '') . ($inputclass ? (' class="' . $inputclass . '"') : '') . '/></td>
		</tr>
		<tr>
			<td class="weEditmodeStyle" colspan="2" style="text-align:left">
				<table class="weEditTable">
					<tr>
						<td style="padding-right: 5px;"><input style="border:0 solid black;" type="checkbox" id="' . we_fileupload::USERINPUT_TYPE_DEL . $name . '" name="' . we_fileupload::USERINPUT_TYPE_DEL . $name . '" value="1" ' . $checked . '/></td>
						<td><label for="' . we_fileupload::USERINPUT_TYPE_DEL . $name . '"' . ($checkboxstyle ? (' style="' . $checkboxstyle . '"') : '') . ($checkboxclass ? (' class="' . $checkboxclass . '"') : '') . '>' . $checkboxtext . '</label></td>
					</tr>
				</table>
			</td>
		</tr>
	</table>';
		}
		$hidden = '<input type="hidden" name="' . we_fileupload::USERINPUT_TYPE_VIDEO . $name . '" value="' . $dataId . '" />';

		if(isset($_SESSION[$dataId]['serverPath'])){
			return $hidden;
		}
		if(!empty($_SESSION[$dataId]['id'])){

			if(!empty($_SESSION[$dataId]['doDelete'])){
				return $hidden;
			}
			$attribs = self::removeAttribs($attribs, ['width', 'height']);
			$attribs['id'] = $_SESSION[$dataId]['id'];
			return self::$doc->getField($attribs, 'video') . $hidden;
		}

		return '';
	}

	protected static function getBinary(array $attribs, bool $editable, string $orgVal, string $size, string $fieldname, string $name): string{
		$dataId = we_base_request::string(we_fileupload::USERINPUT_TYPE_BINARY . $name, md5(uniqid(__CLASS__, true)));
		$bordercolor = self::getAttribute('bordercolor', $attribs, '#006DB8', we_base_request::STRING);
		$checkboxstyle = self::getAttribute('checkboxstyle', $attribs, '', we_base_request::STRING);
		$inputstyle = self::getAttribute('inputstyle', $attribs, '', we_base_request::STRING);
		$checkboxclass = self::getAttribute('checkboxclass', $attribs, '', we_base_request::STRING);
		$inputclass = self::getAttribute('inputclass', $attribs, '', we_base_request::STRING);
		$checkboxtext = self::getAttribute('checkboxtext', $attribs, g_l('parser', '[delete]'), we_base_request::RAW);

		if($editable){
			self::attributeMissing($attribs, ['parentid' => false], __CLASS__);

			if(!isset($_SESSION[$dataId])){
				$_SESSION[$dataId] = [];
			}
			$_SESSION[$dataId]['parentid'] = self::getAttribute('parentid', $attribs, 0, we_base_request::INT);
			$_SESSION[$dataId]['id'] = $orgVal ?: '';
			if($_SESSION[$dataId]['id']){
				$attribs['id'] = $_SESSION[$dataId]['id'];
			}

			if(isset($_SESSION[$dataId]['serverPath']) && !str_contains($_SESSION[$dataId]['serverPath'], TEMP_PATH)){
				$src = we_base_file::addPreSlash(substr($_SESSION[$dataId]['serverPath'], strlen(WE_DOCUMENT_ROOT)) ?: '');
				$tag = '<a href="' . $src . '" target="_blank">' . $src . '</a>';
			} else {
				$binaryTag = self::$doc->getField($attribs, 'binary');
				$t = explode('_', $binaryTag[0]);
				unset($t[1], $t[0]);
				$fn = implode('_', $t);
				$tag = '<a href="' . $binaryTag[1] . '" target="_blank">' . $fn . '</a>';
			}

			$checked = (!empty($_SESSION[$dataId]['doDelete']) ? ' checked' : '');
			$inputstyle = ($size ? 'width:' . $size . 'em;' : '') . $inputstyle;
			return '<table class="we we_userinput weEditTable" style="border-color:' . $bordercolor . ';">
	<tr>
		<td class="weEditmodeStyle" colspan="2" style="text-align:center">' . $tag . '
			<input type="hidden" name="' . we_fileupload::USERINPUT_TYPE_BINARY . $name . '" value="' . $dataId . '" /></td>
	</tr>
	<tr>
		<td class="weEditmodeStyle" colspan="2" style="text-align:left">
			<input name="' . $fieldname . '" type="file" accept="application/*"' . ($inputstyle ? (' style="' . $inputstyle . '"') : '') . ($inputclass ? (' class="' . $inputclass . '"') : '') . '/>
		</td>
	</tr>
	<tr>
		<td class="weEditmodeStyle" colspan="2" style="text-align:left">
			<table class="weEditTable">
				<tr>
					<td style="padding-right: 5px;"><input style="border:0 solid black;" type="checkbox" id="' . we_fileupload::USERINPUT_TYPE_DEL . $name . '" name="' . we_fileupload::USERINPUT_TYPE_DEL . $name . '" value="1" ' . $checked . '/></td>
					<td><label for="' . we_fileupload::USERINPUT_TYPE_DEL . $name . '"' . ($checkboxstyle ? (' style="' . $checkboxstyle . '"') : '') . ($checkboxclass ? (' class="' . $checkboxclass . '"') : '') . '>' . $checkboxtext . '</label></td>
				</tr>
			</table>
		</td>
	</tr>
</table>';
		}
		$hidden = '<input type="hidden" name="' . we_fileupload::USERINPUT_TYPE_BINARY . $name . '" value="' . $dataId . '" />';

		if(isset($_SESSION[$dataId]["serverPath"])){
			//$src = '/' . ltrim(substr($_SESSION[$binaryDataId]['serverPath'], strlen(WE_DOCUMENT_ROOT)) ?: '', '/');
			return $hidden;
		}
		if(empty($_SESSION[$dataId]['id'])){
			return '';
		}
		if(!empty($_SESSION[$dataId]['doDelete'])){
			return $hidden;
		}

		$attribs['id'] = $_SESSION[$dataId]['id'];
		$binaryTag = self::$doc->getField($attribs, 'binary');
		$t = explode('_', $binaryTag[0]);
		unset($t[1], $t[0]);
		$fn = implode('_', $t);
		$tag = '<a href="' . $binaryTag[1] . '" target="_blank">' . $fn . '</a>';
		return we_html_element::span(['class' => 'we we_userinput'], $tag . $hidden);
	}

}
